/***************************************************************************************/
/* Unterprogramme zum Lesen der x- und y-Koordinate von einem resistiven Touchpanel    */
/*                                                                                     */
/*   255 +-------------------------------------+                                       */
/*       |                                     |                                       */  
/*       |                                     |                                       */  
/*       |                                     |                                       */  
/*       |                                     |                                       */  
/*       |                                     |                                       */  
/*       |                                     |                                       */  
/*     0 +-------------------------------------+                                       */
/*       0                                    255                                      */
/*                                                                                     */
/* Compiler: mikroC                                                                    */
/*                                                                                     */
/* Entwickler: Stefan Buchgeher                                                        */
/* Entwicklungsbeginn der Software: 24. Juli 2009                                      */
/* Funktionsfaehig seit: 24. Juli 2009                                                 */
/* Letzte Bearbeitung: 16. August 2009                                                 */
/***************************************************************************************/

/****************** Include-Dateien ****************************************************/
#include "TOUCHPANEL.H"
#include "myPIC_ADC.H"

 
/****************** Unterprogramme zum Lesen der x- und y-Koordinate *******************/

/***************************************************************************************/
/* TP_GetX:                                                                            */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   x-Koordinate von Touchpanel lesen:                                                */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*   + Portpin DRIVEA = 1 und Portpin DRIVEB = 0 (Rechts: +Vcc, Links: Vss,            */
/*     Oben: hochohmig, Unten: x-Koordinate)                                           */
/*   + 5ms warten                                                                      */
/*   + x-Wert (Unten) lesen                                                            */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   keine                                                                             */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   x-Koordinate (0...255)                                                            */
/***************************************************************************************/
char TP_GetX(void)
{
  char x_Koordinate;

  
  // Portpin DRIVEA = 1 und Portpin DRIVEB = 0 (Rechts: +Vcc, Links: Vss,
  // Oben: hochohmig, Unten: x-Koordinate)
  TP_PORT_DRIVEA = 1;         
  TP_PORT_DRIVEB = 0;         

  // 5ms warten
  Delay_ms(5);
 
  // x-Wert (Unten) lesen
  x_Koordinate = ADC8(TP_CHAN_READX);  

  return (x_Koordinate);
}


/***************************************************************************************/
/* TP_GetY:                                                                            */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   y-Koordinate von Touchpanel lesen:                                                */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*   + Portpin DRIVEA = 0 und Portpin DRIVEB = 1 (Oben: +Vcc, Unten: Vss,              */
/*     Rechts: hochohmig, Links: y-Koordinate)                                         */
/*   + 5ms warten                                                                      */
/*   + y-Wert (Links) lesen                                                            */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   keine                                                                             */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   y-Koordinate (0...255)                                                            */
/***************************************************************************************/
char TP_GetY(void)
{
  char y_Koordinate;


  // Portpin DRIVEA = 0 und Portpin DRIVEB = 1 (Oben: +Vcc, Unten: Vss,
  // Rechts: hochohmig, Links: y-Koordinate)
  TP_PORT_DRIVEA = 0;          
  TP_PORT_DRIVEB = 1;          

  // 5ms warten 
  Delay_ms(5);

  // y-Wert (Links) lesen
  y_Koordinate = ADC8(TP_CHAN_READY);  

  return (y_Koordinate);
}