/***************************************************************************************/
/* Touchpanel - Demonstrationsbeispiel                                                 */
/*                                                                                     */
/*   255 +------------------------------------+                                        */
/*   235 |   +----------+      +----------+   |                                        */
/*       |   | Taste 1  |      | Taste 3  |   |                                        */
/*       |   |          |      |          |   |                                        */
/*   148 |   +----------+      +----------+   |                                        */
/*   108 |   +----------+      +----------+   |                                        */
/*       |   | Taste 2  |      | Taste 4  |   |                                        */
/*       |   |          |      |          |   |                                        */
/*    20 |   +----------+      +----------+   |                                        */
/*     0 +------------------------------------+                                        */
/*       0  20         108    148        235 255                                       */
/*                                                                                     */
/* interner Takt (4 MHz)                                                               */
/*                                                                                     */
/* Compiler: mikroC                                                                    */
/*                                                                                     */
/* Entwickler: Stefan Buchgeher                                                        */
/* Entwicklungsbeginn der Software: 15. August 2009                                    */
/* Funktionsfaehig seit: 15. August 2009                                               */
/* Letzte Bearbeitung: 16. August 2009                                                 */
/***************************************************************************************/

/****************** Include-Dateien ****************************************************/
#include "TOUCHPANEL.H"


/****************** Strukturen *********************************************************/
/* keine Strukturen verwendet */


/****************** Externe Register ***************************************************/
/* keine externen Register */


/****************** Bits in den externen Registern *************************************/
/* keine externen Register */


/****************** Globale Register ***************************************************/
char    FlagISRHP;
char    ZaehlerZeitbasis100ms;


/****************** Bits in den globalen Registern *************************************/
/* Register FlagISRHP */
#define FlagZeitbasis100ms       FlagISRHP.F0


/****************** Portbelegung *******************************************************/
/* Siehe TOUCHPANEL.H */

/* Leuchtdioden */
#define LED_Taste1               PORTB.F7
#define LED_Taste2               PORTB.F5
#define LED_Taste3               PORTB.F6
#define LED_Taste4               PORTB.F4


/****************** Konstanten *********************************************************/
// Konstanten fuer die Zeitbasen
#define KONSTZEITBASIS100MS      12

// Konstanten fuer die Tastenkoordinaten (Demo)
// Taste 1
#define KONSTTASTE1X1            20
#define KONSTTASTE1X2            108
#define KONSTTASTE1Y1            148
#define KONSTTASTE1Y2            235
// Taste 2
#define KONSTTASTE2X1            20
#define KONSTTASTE2X2            108
#define KONSTTASTE2Y1            20
#define KONSTTASTE2Y2            108
// Taste 3
#define KONSTTASTE3X1            148
#define KONSTTASTE3X2            235
#define KONSTTASTE3Y1            148
#define KONSTTASTE3Y2            235
// Taste 4
#define KONSTTASTE4X1            148
#define KONSTTASTE4X2            235
#define KONSTTASTE4Y1            20
#define KONSTTASTE4Y2            108


/****************** Tabellen ***********************************************************/
/* keine Tabellen */


/****************** Funktionsprototypen ************************************************/
void Init(void);
void TouchPanelTastenRoutine(void);


/****************** ISR - Timer0 *******************************************************/

/***************************************************************************************/
/* Interrupt Service Routine:                                                          */
/*                                                                                     */
/* Aufruf:                                                                             */
/*   alle 8.2 ms (ungefaehr)                                                           */
/*                                                                                     */
/* Aufgaben:                                                                           */
/*   + Zeitbasis fuer 100 Millisekunde erzeugen                                        */
/*   + Das Timer-Interrupt-Flag T0IF wieder loeschen                                   */
/***************************************************************************************/
void interrupt (void)                   // Interruptroutine
{
    ZaehlerZeitbasis100ms--;
    if (ZaehlerZeitbasis100ms == 0)     // Botschaftsflag fuer 100ms-Zeitbasis setzen
    {
        FlagZeitbasis100ms = 1;
        ZaehlerZeitbasis100ms = KONSTZEITBASIS100MS; // Zaehlregister fuer 100-ms-
                                        // Zeitbasis neu laden
    }
    INTCON.T0IF = 0;                   // Timer-Interrupt-Flag T0IF wieder loeschen
}


/****************** Unterprogramme und Funktionen **************************************/

/***************************************************************************************/
/* Init:                                                                               */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   Initialisierung des Prozessor:                                                    */
/*      + Timer 0 (TMR0) loeschen                                                      */
/*      + Timer 0-ISR soll ca. alle 8,2ms aufgerufen werden, daher den Vorteiler mit   */
/*        1:32 laden (bei einem 4-MHz-Takt)                                            */
/*                          4  * 256 * VT   4 * 256 * 32                               */
/*        ISR-Aufruf [us] = ------------- = ------------ = 8192 us = 8,2 ms            */
/*                             f [MHz]           4                                     */
/*      + internen Oszillator konfigurieren (hier: 4MHz)                               */
/*      + Port A fuer das Touchpanel als Eingang (fuer ADC) bzw.als Ausgang definieren */
/*      + Port B fuer die Leuchtdioden als Ausgang definieren und Inhalt loeschen      */
/*      + ADC konfigurieren                                                            */
/*      + Zaehlregister fuer Zeitbasis initialisieren                                  */
/*      + Interrupt freigeben (Timer0 freigeben durch Setzen von GIE und T0IE im       */
/*        INTCON-Register)                                                             */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   keiner                                                                            */
/***************************************************************************************/
void Init(void)
{
    // Timer-0-Interrupt konfigurieren
    TMR0 = 0;                        // Timer 0 auf 0 voreinstellen
    OPTION_REG = 0b10000100;         // Option-Register (Bank 1)
/*                 +---------------- Bit 7 (nRBPU): Pull-Up-Widerstaende am Port B
                    |||||||               0 : Pull-Up aktiviert
                    |||||||          ->   1 : Pull-Up deaktiviert
                    +---------------- Bit 6 (INTEDG): Interrupt Edge Select Bit
                     ||||||          ->   0 : Interrupt on falling edge of RB0/INT pin
                     ||||||               1 : Interrupt on rising edge of RB0/INT pin
                     +--------------- Bit 5 (T0CS): Timer 0 Clock source Select Bit
                      |||||          ->   0 : Transition on RA4/T0CKI pin
                      |||||               1 : Internal instruction cycle clock (CLKOUT)
                      +-------------- Bit 4 (T0SE): TMR0 Source Edge Select bit
                       ||||          ->   0 : Increment on low-to-high tranition
                       ||||               1 : Increment on high-to-low tranition
                       +------------ Bit 3 (PSA): Prescaler Assignment bit
                        |||          ->   0 : Prescaler is assigned to the TIMER0 module
                        |||               1 : Prescaler is assigned to the WDT
                        +++--------- Bit 2-0 (PS2:PS0): Prescaler Rate Select bits
                                       Bit Value        TMR0 Rate         WDT Rate
                                         000 :             1:2               1:1
                                         001 :             1:4               1:2
                                         010 :             1:8               1:4
                                         011 :             1:16              1:8
                                     ->  100 :             1:32              1:16
                                         101 :             1:64              1:32
                                         110 :             1:128             1:64
                                         111 :             1:256             1:128
*/
    // internen Oszillator konfigurieren
    OSCCON = 0b01101010;             // Oszillator-Control-Register (Bank 1)
/*             +-------------------- Bit 7 Reserve
                +++----------------- Bit 6-4 (IRCF2:IRCF0): Internal RC-Oscillator
                   ||||                                      Frequency Select bits
                   ||||                  000: 31.25 kHz (INTRC source drives clock)
                   ||||                  001: 125 kHz
                   ||||                  010: 250 kHz
                   ||||                  011: 500 kHz
                   ||||                  100: 1 MHz
                   ||||                  101: 2 MHz
                   ||||              ->  110: 4 MHz
                   ||||                  111: 8 MHz
                   +---------------- Bit 3 (OSTS): Oscillator Start-up Time-out
                    |||                             Status bit
                    |||                   0 : Device is running from T1OSC or INTRC
                    |||                       as a secondary system clock
                    |||              ->   1 : Device is running from the primary
                    |||                       system clock
                    +--------------- Bit 2 (IOFS): INTOSC Frequency Stable bit
                     ||              ->   0 : Frequency is not stable
                     ||                   1 : Frequency is stable
                     ++------------- Bit 1,0 (SCS1:SCS0): Oscillator Mode Select bits
                                         00: Oscillator mode defined by Fosc<2:0>
                                         01: T1OSC is used for system clock
                                     ->  10: internal RC is used for system clock
                                         11: Reserved
*/

    // Ports konfigurieren
    TRISA = 0b00111100;              // Richtungsregister Port A (0: Ausgang, 1: Eingang)
/*            ++-------------------- Bit 7-6 Reserve
                +------------------- Bit 5 Port RA5: hier unbenutzt
                 +------------------ Bit 4 Port RA4: hier unbenutzt
                  +----------------- Bit 3 Port RA3: hier analoger Eingang (Touchpanel)
                   +---------------- Bit 2 Port RA2: hier analoger Eingang (Touchpanel)
                    +--------------- Bit 1 Port RA1: hier hier Ausgang (Touchpanel)
                     +-------------- Bit 0 Port RA0: hier hier Ausgang (Touchpanel)
*/

    TRISB = 0b00000000;              // Richtungsregister Port B (0: Ausgang, 1: Eingang)
/*            +--------------------- Bit 7 Port RB7: hier Ausgang (LED Taste 1)
               +-------------------- Bit 6 Port RB6: hier Ausgang (LED Taste 3)
                +------------------- Bit 5 Port RB5: hier Ausgang (LED Taste 2)
                 +------------------ Bit 4 Port RB4: hier Ausgang (LED Taste 4)
                  +----------------- Bit 3 Port RB3: hier unbenutzt
                   +---------------- Bit 2 Port RB2: hier unbenutzt
                    +--------------- Bit 1 Port RB1: hier unbenutzt
                     +-------------- Bit 0 Port RB0: hier unbenutzt
*/

    PORTB = 0;

    // ADC konfigurieren
    ANSEL = 0b00001100;              // ANSEL-Register (0: Digital I/O, 1: Analog I/O)
/*            +--------------------  Bit 7 Reserve
               +-------------------- Bit 6 Port RB7/AN6: hier Digital I/O
                +------------------- Bit 5 Port RB6/AN5: hier Digital I/O
                 +------------------ Bit 4 Port RA4/AN4: hier unbenutzt
                  +----------------- Bit 3 Port RA3/AN3: hier Analog I/O
                   +---------------- Bit 2 Port RA2/AN2: hier Analog I/O
                    +--------------- Bit 1 Port RA1/AN1: hier Digital I/O
                     +-------------- Bit 0 Port RA0/AN0: hier Digital I/O
*/

    ADCON0 = 0b01000001;             // ADC-Control-Register 0 (Bank 1)
/*             ++------------------- Bit 7-6 (ADCS1:ADCS0): A/D Conersion Clock Select
                 ||||||                  bits
                 ||||||                 IF ADSC2 = 0:
                 ||||||                  00 : Fosc/2   (bei fosz: 0 - 1.25 MHz)
                 ||||||              ->  01 : Fosc/8   (bei fosz: 1.25 - 5 MHz)
                 ||||||                  10 : Fosc/32  (bei fosz: 5 - 20 MHz)
                 ||||||                  11 : Frc      (bei RC-Oszillator)
                 ||||||                 IF ADSC2 = 1:
                 ||||||                  00 : Fosc/4   (bei fosz: 0 - 1.25 MHz)
                 ||||||                  01 : Fosc/16  (bei fosz: 1.25 - 5 MHz)
                 ||||||                  10 : Fosc/64  (bei fosz: 5 - 20 MHz)
                 ||||||                  11 : Frc      (bei RC-Oszillator)
                 +++---------------- Bit 5-3 (CHS2:CHS0): Analog Channel Select bits
                    |||              ->  000: channel 0 (RA0/AN0)
                    |||                  001: channel 1 (RA1/AN1)
                    |||                  010: channel 2 (RA2/AN2)
                    |||                  011: channel 3 (RA3/AN3)
                    |||                  100: channel 4 (RA4/AN4)
                    |||                  101: channel 5 (RB6/AN5)
                    |||                  110: channel 6 (RB7/AN6)
                    +--------------- Bit 2 (GO/DONE): A/D Conversion Status Bit
                     ||                 IF ADON = 1:
                     ||              ->   0 : A/D conversion not in progress
                     ||                   1 : A/D conversion in progress
                     +-------------- Bit 1 Reserve
                      +------------- Bit 0 (ADON): A/D On Bit
                                          0 : A/D converter module is shut-off
                                     ->   1 : A/D converter module is operating
*/

    ADCON1 = 0b00000010;             // ADC-Control-Register 1 (Bank 1)
/*             +-------------------- Bit 7 (ADFM): A/D Result Formated Select bits
                |||||||              ->   0 : Left justified:
                |||||||                       in ADRESH:  Bit 9 bis Bit 2
                |||||||                       in ADRESL:  Bit 1, Bit 0, Rest 0
                |||||||                   1 : Right justified:
                |||||||                       in ADRESH:  0 0 0 0 0 0 Bit 9 Bit 8
                |||||||                       in ADRESL:  Bit 7 bis Bit 0
                +------------------- Bit 6 (ADCS2): A/D Clock Divide by 2 Select bit
                 ||||||              ->   0 : Disabled
                 ||||||                   1 : A/D clock source is divide by 2 when
                 ||||||                       system clock is used
                 ++----------------- Bit 5,4 (VCFG1:VCFG0): A/D Voltage Reference
                   ||||                          Vref+     Vref-
                   ||||                  00 :    AVdd      AVss
                   ||||                  01 :    AVdd      Vref-
                   ||||                  10 :    Vref+     AVss
                   ||||                  11 :    Vref+     Vref-
                   ++++------------- Bit 3-0 Reserve
*/

    // Zaehlregister fuer Zeitbasis initialisieren
    ZaehlerZeitbasis100ms = KONSTZEITBASIS100MS; // Zaehlregister fuer 100-ms-Zeitbasis
                                               // initialisieren

    // Interrupt freigeben
    INTCON = 0b10100000;        // Interrupt-Control-Register (Bank 1)
/*             +-------------------- Bit 7 (GIE): Global Interrupt Enable bit
                |||||||                   0 : Disables all interrupts
                |||||||              ->   1 : Enables all unmasked interrupts
                +------------------- Bit 6 (PEIE): Peripheral Interrupt Enable bit
                 ||||||              ->   0 : Disables all peripheral interrupts
                 ||||||                   1 : Enables all unmasked peripheral interrupts
                 +------------------ Bit 5 (T0IE): Timer 0 Overflow Interrupt Enable Bit
                  |||||                   0 : Disabled
                  |||||              ->   1 : Enabled
                  +----------------- Bit 4 (INTE): RB0/INT External Interrupt Enable Bit
                   ||||              ->   0 : Disabled
                   ||||                   1 : Enabled
                   +---------------- Bit 3 (RPIE): RB Port Change Interrupt Enable Bit
                    |||              ->   0 : Disabled
                    |||                   1 : Enabled
                    +--------------- Bit 2 (T0IF): Timer 0 Overflow Interrupt Flag bit
                     ||              ->   0 : TMR0 register did not overflow
                     ||                   1 : TMR0 register has overflowed
                     +-------------- Bit 1 (INTF): RB0/INT External Interrupt Flag bit
                      |              ->   0 : RB0/INT external interrupt did not occur
                      |                   1 : RB0/INT external interrupt occurred
                      +------------- Bit 0 (RBIF): RB Port Change Interrupt Flag bit
                                     ->   0 : None of RB7:RB4 pins have changed state
                                          1 : One of the RB7:RB4 pins changed state
*/
}


/***************************************************************************************/
/* TouchPanelTastenRoutine:                                                            */
/*                                                                                     */
/* Aufruf:                                                                             */
/*   ca. alle 100 ms (vom Hauptprogramm)                                               */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   Ermitteln, ob und welche der vier "virtuellen" Tasten am Touch-Panel gedrueckt    */
/*   wurde, und den "virtuellen" Tastenzustand toggeln und den Status mit den LEucht-  */
/*   dioden anzeigen.                                                                  */
/*                                                                                     */
/* Zuordnung der "virtuellen" Tasten am Touch-Panel:                                   */
/*   255 +------------------------------------+                                        */
/*   235 |   +----------+      +----------+   |                                        */
/*       |   | Taste 1  |      | Taste 3  |   |                                        */
/*       |   |          |      |          |   |                                        */
/*   148 |   +----------+      +----------+   |                                        */
/*   108 |   +----------+      +----------+   |                                        */
/*       |   | Taste 2  |      | Taste 4  |   |                                        */
/*       |   |          |      |          |   |                                        */
/*    20 |   +----------+      +----------+   |                                        */
/*     0 +------------------------------------+                                        */
/*       0  20         108    148        235 255                                       */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*   + x- und y-Koordinaten von Touchpanel lesen                                       */
/*   + Pruefen, ob die x- und y-Koordinate im Bereich von Taste 1 liegt, wenn ja, den  */
/*     Zustand der Taste 1 bei einem Uebergang von "nicht gedrueckt" zu "gedrueckt"    */
/*     invertieren (toggeln) und je nach Aufgabe der Taste diese ausfhren (hier: den  */
/*     Status der Taste mit einer Leuchtdiode anzeigen                                 */
/*   + Das selbe nun fuer die anderen drei Tasten                                      */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Anmerkungen:                                                                        */
/*   + Bei dem hier verwendeten resistiven Touch-Panel kann immer nur eine Taste ge-   */
/*     drueckt werden. Mehrere Tasten gleichzeitig (also eine Tastenkombination) ist   */
/*     bei resistiven Touchpanels nicht mglich!                                       */
/*   + das statische Register button_status diehnt hier als Hilfsregister, wobei die   */
/*     Bits folgende Bedeutung haben:                                                  */
/*        Bit 0: Zustand von Taste 1 (0 = Taste nicht gedrueckt, 1 = Taste gedrueckt)  */
/*        Bit 1: Hilfsflag zur Erkennung einer "nicht gedrueckt-gedrueckt-Flanke" von  */
/*               Taste 1                                                               */
/*        Bit 2: Zustand von Taste 2 (0 = Taste nicht gedrueckt, 1 = Taste gedrueckt)  */
/*        Bit 3: Hilfsflag zur Erkennung einer "nicht gedrueckt-gedrueckt-Flanke" von  */
/*               Taste 2                                                               */
/*        Bit 4: Zustand von Taste 3 (0 = Taste nicht gedrueckt, 1 = Taste gedrueckt)  */
/*        Bit 5: Hilfsflag zur Erkennung einer "nicht gedrueckt-gedrueckt-Flanke" von  */
/*               Taste 3                                                               */
/*        Bit 6: Zustand von Taste 4 (0 = Taste nicht gedrueckt, 1 = Taste gedrueckt)  */
/*        Bit 7: Hilfsflag zur Erkennung einer "nicht gedrueckt-gedrueckt-Flanke" von  */
/*               Taste 4                                                               */
/*   + Die zulaessigen Bereiche der Tasten am Touch-Panel sind durch die Konstanten    */
/*     KONSTTASTExXx und KONSTTASTExYx begrenzt.                                       */
/***************************************************************************************/
void TouchPanelTastenRoutine(void)
{
    char x, y;
    static char button_status = 0;


    // Schritt 1: x- und y-Koordinaten von Touchpanel lesen
    x = TP_GetX();
    y = TP_GetY();

    // Schritt 2: Taste 1
    if ((x >= KONSTTASTE1X1) && (x <= KONSTTASTE1X2) && (y >= KONSTTASTE1Y1) && (y <= KONSTTASTE1Y2))
    {
        if (button_status.F1 == 0)
        {
            button_status.F1 = 1;

            if (button_status.F0 == 0)
            {
                button_status.F0 = 1;
                LED_Taste1 = 1;
            }
            else
            {
                button_status.F0 = 0;
                LED_Taste1 = 0;
            }
        }
    }
    else
    {
        button_status.F1 = 0;
    }

    // Schritt 3: Taste 2
    if ((x >= KONSTTASTE2X1) && (x <= KONSTTASTE2X2) && (y >= KONSTTASTE2Y1) && (y <= KONSTTASTE2Y2))
    {
        if (button_status.F2 == 0)
        {
            button_status.F2 = 1;

            if (button_status.F3 == 0)
            {
                button_status.F3 = 1;
                LED_Taste2 = 1;
            }
            else
            {
                button_status.F3 = 0;
                LED_Taste2 = 0;
            }
        }
    }
    else
    {
        button_status.F2 = 0;
    }
    
    // Schritt 4: Taste 3
    if ((x >= KONSTTASTE3X1) && (x <= KONSTTASTE3X2) && (y >= KONSTTASTE3Y1) && (y <= KONSTTASTE3Y2))
    {
        if (button_status.F4 == 0)
        {
            button_status.F4 = 1;

            if (button_status.F5 == 0)
            {
                button_status.F5 = 1;
                LED_Taste3 = 1;
            }
            else
            {
                button_status.F5 = 0;
                LED_Taste3 = 0;
            }
        }
    }
    else
    {
        button_status.F4 = 0;
    }
    
    // Schritt 5: Taste 4
    if ((x >= KONSTTASTE4X1) && (x <= KONSTTASTE4X2) && (y >= KONSTTASTE4Y1) && (y <= KONSTTASTE4Y2))
    {
        if (button_status.F6 == 0)
        {
            button_status.F6 = 1;

            if (button_status.F7 == 0)
            {
                button_status.F7 = 1;
                LED_Taste4 = 1;
            }
            else
            {
                button_status.F7 = 0;
                LED_Taste4 = 0;
            }
        }
    }
    else
    {
        button_status.F6 = 0;
    }
}

/****************** Hauptprogramm ******************************************************/

/***************************************************************************************/
/* Aufgaben des Hauptprogramms:                                                        */
/*   + Controller initialisieren (Unterprogramm INIT)                                  */
/*   + Taetigkeiten/Unterprogramme, die alle zyklisch durchgefuehrt werden muessen:    */
/*      + alle 100 ms: Touchpanel abfragen, ob eine "virtuelle" Taste gedrueckt wurde  */
/*        (Unterprogramm TouchPanelTastenRoutine)                                      */
/***************************************************************************************/
void main(void)
{
    // Controller initialisieren
    Init();

    // Endlosschleife
    while(1)
    {
        if (FlagZeitbasis100ms)
        {
            // Touchpanel abfragen, ob eine "virtuelle" Taste gedrueckt wurde
            TouchPanelTastenRoutine();
            
            // Botschaftsflag (fuer 100ms-Zeitbasis) zuruecksetzen
            FlagZeitbasis100ms = 0;
        }
    }
}





