/***************************************************************************************/
/* Touchscreen - Demonstrationsbeispiel                                                */
/*                                                                                     */
/*   255 +-------------------------------------+                                       */
/*       |                                     |                                       */
/*   184 |   +----------+      +----------+    |                                       */
/*       |   |          |      |          |    |                                       */
/*       |   | Taste 1  |      | Taste 2  |    |                                       */
/*       |   |          |      |          |    |                                       */
/*    72 |   +----------+      +----------+    |                                       */
/*       |                                     |                                       */
/*     0 +-------------------------------------+                                       */
/*       0  20         116    140        236  255                                      */
/*                                                                                     */
/* externer Takt (20 MHz)                                                              */
/*                                                                                     */
/* Compiler: mikroC                                                                    */
/*                                                                                     */
/* Entwickler: Stefan Buchgeher                                                        */
/* Entwicklungsbeginn der Software: 24. Juli 2009                                      */
/* Funktionsfaehig seit: 24. Juli 2009                                                 */
/* Letzte Bearbeitung: 16. August 2009                                                 */
/***************************************************************************************/

/****************** Include-Dateien ****************************************************/
#include "TOUCHPANEL.H"


/****************** Strukturen *********************************************************/
/* keine Strukturen verwendet */


/****************** Externe Register ***************************************************/
/* keine externen Register */


/****************** Bits in den externen Registern *************************************/
/* keine externen Register */


/****************** Globale Register ***************************************************/
char    FlagISRHP;
char    ZaehlerZeitbasis100ms;


/****************** Bits in den globalen Registern *************************************/
/* Register FlagISRHP */
#define FlagZeitbasis100ms      FlagISRHP.F0


/****************** Portbelegung *******************************************************/
/* Siehe TOUCHPANEL.H */


/****************** Konstanten *********************************************************/
// Konstanten fuer die Zeitbasen
#define KONSTZEITBASIS100MS      15

// Konstanten fuer die Tastenkoordinaten (Demo)
// Taste 1
#define KONSTTASTE1X1            20
#define KONSTTASTE1X2            116
#define KONSTTASTE1Y1            72
#define KONSTTASTE1Y2            184
// Taste 2
#define KONSTTASTE2X1            140
#define KONSTTASTE2X2            236
#define KONSTTASTE2Y1            72
#define KONSTTASTE2Y2            184


/****************** Tabellen ***********************************************************/
/* keine Tabellen */


/****************** Funktionsprototypen ************************************************/
void Init(void);
void TouchPanelTastenRoutine(void);


/****************** ISR - Timer0 *******************************************************/

/***************************************************************************************/
/* Interrupt Service Routine:                                                          */
/*                                                                                     */
/* Aufruf:                                                                             */
/*   alle 6.5 ms (ungefaehr)                                                           */
/*                                                                                     */
/* Aufgaben:                                                                           */
/*   + Zeitbasis fuer 100 Millisekunde erzeugen                                        */
/*   + Das Timer-Interrupt-Flag T0IF wieder loeschen                                   */
/***************************************************************************************/
void interrupt (void)                   // Interruptroutine
{
    ZaehlerZeitbasis100ms--;
    if (ZaehlerZeitbasis100ms == 0)     // Botschaftsflag fuer 100ms-Zeitbasis setzen
    {
        FlagZeitbasis100ms = 1;
        ZaehlerZeitbasis100ms = KONSTZEITBASIS100MS; // Zaehlregister fuer 100-ms-
                                        // Zeitbasis neu laden
    }
    INTCON.T0IF = 0;                   // Timer-Interrupt-Flag T0IF wieder loeschen
}


/****************** Unterprogramme und Funktionen **************************************/

/***************************************************************************************/
/* Init:                                                                               */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   Initialisierung des Prozessor:                                                    */
/*      + Timer 0 (TMR0) loeschen                                                      */
/*      + Timer 0-ISR soll ca. alle 6,5ms aufgerufen werden, daher den Vorteiler mit   */
/*        1:128 laden (bei einem 20-MHz-Takt)                                          */
/*                          4  * 256 * VT   4 * 256 * 128                              */
/*        ISR-Aufruf [us] = ------------- = ------------- = 6553.6 us = 6.55 ms        */
/*                             f [MHz]           20                                    */
/*      + Port A und C fuer das Touchpanel als Eingang (fuer ADC) bzw.als Ausgang      */
/*        definieren (Anmerkung: Port B und Port D werden fuer das Grafik-Display im   */
/*        Hauptprogramm definiert!)                                                    */
/*      + ADC konfigurieren                                                            */
/*      + Zaehlregister fuer Zeitbasis initialisieren                                  */
/*      + Interrupt freigeben (Timer0 freigeben durch Setzen von GIE und T0IE im       */
/*        INTCON-Register)                                                             */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   keiner                                                                            */
/***************************************************************************************/
void INIT(void)
{
    // Timer-0-Interrupt konfigurieren
    TMR0 = 0;                        // Timer 0 auf 0 voreinstellen
    OPTION_REG = 0b10000110;         // Option-Register (Bank 1)
/*                 +---------------- Bit 7 (nRBPU): Pull-Up-Widerstaende am Port B
                    |||||||               0 : Pull-Up aktiviert
                    |||||||          ->   1 : Pull-Up deaktiviert
                    +---------------- Bit 6 (INTEDG): Interrupt Edge Select Bit
                     ||||||          ->   0 : Interrupt on falling edge of RB0/INT pin
                     ||||||               1 : Interrupt on rising edge of RB0/INT pin
                     +--------------- Bit 5 (T0CS): Timer 0 Clock source Select Bit
                      |||||          ->   0 : Transition on RA4/T0CKI pin
                      |||||               1 : Internal instruction cycle clock (CLKOUT)
                      +-------------- Bit 4 (T0SE): TMR0 Source Edge Select bit
                       ||||          ->   0 : Increment on low-to-high tranition
                       ||||               1 : Increment on high-to-low tranition
                       +------------ Bit 3 (PSA): Prescaler Assignment bit
                        |||          ->   0 : Prescaler is assigned to the TIMER0 module
                        |||               1 : Prescaler is assigned to the WDT
                        +++--------- Bit 2-0 (PS2:PS0): Prescaler Rate Select bits
                                       Bit Value        TMR0 Rate         WDT Rate
                                         000 :             1:2               1:1
                                         001 :             1:4               1:2
                                         010 :             1:8               1:4
                                         011 :             1:16              1:8
                                         100 :             1:32              1:16
                                         101 :             1:64              1:32
                                     ->  110 :             1:128             1:64
                                         111 :             1:256             1:128
*/

    // Ports konfigurieren
    TRISA = 0b00111111;              // Richtungsregister Port A (0: Ausgang, 1: Eingang)
/*            ++-------------------- Bit 7-6 Reserve
                +------------------- Bit 5 Port RA5: hier unbenutzt
                 +------------------ Bit 4 Port RA4: hier unbenutzt
                  +----------------- Bit 3 Port RA3: hier unbenutzt
                   +---------------- Bit 2 Port RA2: hier unbenutzt
                    +--------------- Bit 1 Port RA1: hier analoger Eingang (Touchpanel)
                     +-------------- Bit 0 Port RA0: hier analoger Eingang (Touchpanel)
*/

/* TRISB und TRISD (fuer Grafik-Display): Siehe Hauptprogramm */
/* (Initialisierung des Display) */

    TRISC = 0b00000000;              // Richtungsregister Port C (0: Ausgang, 1: Eingang)
/*            +--------------------- Bit 7 Port RC7: hier unbenutzt
               +-------------------- Bit 6 Port RC6: hier unbenutzt
                +------------------- Bit 5 Port RC5: hier unbenutzt
                 +------------------ Bit 4 Port RC4: hier unbenutzt
                  +----------------- Bit 3 Port RC3: hier unbenutzt
                   +---------------- Bit 2 Port RC2: hier unbenutzt
                    +--------------- Bit 1 Port RC1: hier Ausgang (Touchpanel)
                     +-------------- Bit 0 Port RC0: hier Ausgang (Touchpanel)
*/



    // ADC konfigurieren
    ADCON0 = 0b10000001;             // ADC-Control-Register 0 (Bank 1)
/*             ++------------------- Bit 7-6 (ADCS1:ADCS0): A/D Conersion Clock Select
                 ||||||                  bits
                 ||||||                  00 : Fosc/2   (bei fosz: 0 - 1.25 MHz)
                 ||||||                  01 : Fosc/8   (bei fosz: 1.25 - 5 MHz)
                 ||||||              ->  10 : Fosc/32..(bei fosz: 5 - 20 MHz)
                 ||||||                  11 : Frc      (bei RC-Oszillator)
                 +++---------------- Bit 5-3 (CHS2:CHS0): Analog Channel Select bits
                    |||              ->  000: channel 0 (RA0/AN0)
                    |||                  001: channel 1 (RA1/AN1)
                    |||                  010: channel 2 (RA2/AN2)
                    |||                  011: channel 3 (RA3/AN3)
                    |||                  100: channel 4 (RA5/AN4)
                    |||                  101: channel 5 (RE0/AN5)
                    |||                  110: channel 6 (RE1/AN6)
                    |||                  111: channel 7 (RE2/AN7)
                    +--------------- Bit 2 (GO/DONE): A/D Conversion Status Bit
                     ||                 IF ADON = 1:
                     ||              ->   0 : A/D conversion not in progress
                     ||                   1 : A/D conversion in progress
                     +-------------- Bit 1 Reserve
                      +------------- Bit 0 (ADON): A/D On Bit
                                          0 : A/D converter module is shut-off
                                     ->   1 : A/D converter module is operating
*/

    ADCON1 = 0b00000010;             // ADC-Control-Register 1 (Bank 1)
/*             +-------------------- Bit 7 (ADFM): A/D Result Formated Select bits
                |||||||              ->   0 : Left justified:
                |||||||                       in ADRESH:  Bit 9 bis Bit 2
                |||||||                       in ADRESL:  Bit 1, Bit 0, Rest 0
                |||||||                   1 : Right justified:
                |||||||                       in ADRESH:  0 0 0 0 0 0 Bit 9 Bit 8
                |||||||                       in ADRESL:  Bit 7 bis Bit 0
                +++----------------- Bit 6-4 Reserve
                   ++++------------- Bit 3-0 (PCFG3:PCFG0): A/D Port Configuration
                                              Control bits
                                        RE2 RE1 RE0 RA5  RA3   RA2  RA1 RA0 Vref+ Vref-
                                  0000:  A   A   A   A    A     A    A   A   Vdd   Vss
                                  0001:  A   A   A   A   Vref+  A    A   A   RA3   Vss
                               -> 0010:  D   D   D   A    A     A    A   A   Vdd   Vss
                                  0011:  D   D   D   A   Vref+  A    A   A   RA3   Vss
                                  0100:  D   D   D   D    A     D    A   A   Vdd   Vss
                                  0101:  D   D   D   D   Vref+  D    A   A   RA3   Vss
                                  011x:  D   D   D   D    D     D    D   D   Vdd   Vss
                                  1000:  A   A   A   A   Vref+ Vref- A   A   RA3   RA2
                                  1001:  D   D   A   A    A     A    A   A   Vdd   Vss
                                  1010:  D   D   A   A   Vref+  A    A   A   RA3   Vss
                                  1011:  D   D   A   A   Vref+ Vref- A   A   RA3   RA2
                                  1100:  D   D   D   A   Vref+ Vref- A   A   RA3   RA2
                                  1101:  D   D   D   D   Vref+ Vref- A   A   RA3   RA2
                                  1110:  D   D   D   D    D     D    D   A   Vdd   Vss
                                  1111:  D   D   D   D   Vref+ Vref- D   A   RA3   RA2
*/
    // Zaehlregister fuer Zeitbasis initialisieren
    ZaehlerZeitbasis100ms = KONSTZEITBASIS100MS; // Zaehlregister fuer 100-ms-Zeitbasis
                                               // initialisieren

    // Interrupt freigeben
    INTCON = 0b10100000;        // Interrupt-Control-Register (Bank 1)
/*             +-------------------- Bit 7 (GIE): Global Interrupt Enable bit
                |||||||                   0 : Disables all interrupts
                |||||||              ->   1 : Enables all unmasked interrupts
                +------------------- Bit 6 (PEIE): Peripheral Interrupt Enable bit
                 ||||||              ->   0 : Disables all peripheral interrupts
                 ||||||                   1 : Enables all unmasked peripheral interrupts
                 +------------------ Bit 5 (T0IE): Timer 0 Overflow Interrupt Enable Bit
                  |||||                   0 : Disabled
                  |||||              ->   1 : Enabled
                  +----------------- Bit 4 (INTE): RB0/INT External Interrupt Enable Bit
                   ||||              ->   0 : Disabled
                   ||||                   1 : Enabled
                   +---------------- Bit 3 (RPIE): RB Port Change Interrupt Enable Bit
                    |||              ->   0 : Disabled
                    |||                   1 : Enabled
                    +--------------- Bit 2 (T0IF): Timer 0 Overflow Interrupt Flag bit
                     ||              ->   0 : TMR0 register did not overflow
                     ||                   1 : TMR0 register has overflowed
                     +-------------- Bit 1 (INTF): RB0/INT External Interrupt Flag bit
                      |              ->   0 : RB0/INT external interrupt did not occur
                      |                   1 : RB0/INT external interrupt occurred
                      +------------- Bit 0 (RBIF): RB Port Change Interrupt Flag bit
                                     ->   0 : None of RB7:RB4 pins have changed state
                                          1 : One of the RB7:RB4 pins changed state
*/
}


/***************************************************************************************/
/* TouchPanelTastenRoutine:                                                            */
/*                                                                                     */
/* Aufruf:                                                                             */
/*   ca. alle 100 ms (vom Hauptprogramm)                                               */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   Ermitteln, ob und welche der beiden "virtuellen" Tasten am Touch-Panel gedrueckt  */
/*   wurde, und den "virtuellen" Tastenzustand toggeln und den Status am Grafik-       */
/*   Display anzeigen.                                                                 */
/*                                                                                     */
/* Zuordnung der "virtuellen" Tasten am Touch-Panel:                                   */
/*   255 +-------------------------------------+                                       */
/*       |                                     |                                       */
/*   184 |   +----------+      +----------+    |                                       */
/*       |   |          |      |          |    |                                       */
/*       |   | Taste 1  |      | Taste 2  |    |                                       */
/*       |   |          |      |          |    |                                       */
/*    72 |   +----------+      +----------+    |                                       */
/*       |                                     |                                       */
/*     0 +-------------------------------------+                                       */
/*       0  20         116    140        236  255                                      */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*   + x- und y-Koordinaten von Touchpanel lesen                                       */
/*   + Pruefen, ob die x- und y-Koordinate im Bereich von Taste 1 liegt, wenn ja, den  */
/*     Zustand der Taste 1 bei einem Uebergang von "nicht gedrueckt" zu "gedrueckt"    */
/*     invertieren (toggeln) und je nach Aufgabe der Taste diese ausfhren (hier: den  */
/*     Status der Taste am Grafik-Display als Text anzeigen                            */
/*   + Das selbe nun fuer die zweite Taste                                             */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Anmerkungen:                                                                        */
/*   + Bei dem hier verwendeten resistiven Touch-Panel kann immer nur eine Taste ge-   */
/*     drueckt werden. Mehrere Tasten gleichzeitig (also eine Tastenkombination) ist   */
/*     bei resistiven Touchpanels nicht mglich!                                       */
/*   + das statische Register button_status diehnt hier als Hilfsregister, wobei die   */
/*     Bits 0 bis 4 folgende Bedeutung haben:                                          */
/*        Bit 0: Zustand von Taste 1 (0 = Taste nicht gedrueckt, 1 = Taste gedrueckt)  */
/*        Bit 1: Hilfsflag zur Erkennung einer "nicht gedrueckt-gedrueckt-Flanke" von  */
/*               Taste 1                                                               */
/*        Bit 2: Zustand von Taste 2 (0 = Taste nicht gedrueckt, 1 = Taste gedrueckt)  */
/*        Bit 3: Hilfsflag zur Erkennung einer "nicht gedrueckt-gedrueckt-Flanke" von  */
/*               Taste 2                                                               */
/*   + Die zulaessigen Bereiche der Tasten am Touch-Panel sind durch die Konstanten    */
/*     KONSTTASTExXx und KONSTTASTExYx begrenzt.                                       */
/***************************************************************************************/
void TouchPanelTastenRoutine(void)
{
    char x, y;
    static char button_status = 0;


    // Schritt 1: x- und y-Koordinaten von Touchpanel lesen
    x = TP_GetX();
    y = TP_GetY();

    // Schritt 2: Taste 1
    if ((x >= KONSTTASTE1X1) && (x <= KONSTTASTE1X2) && (y >= KONSTTASTE1Y1) && (y <= KONSTTASTE1Y2))
    {
        if (button_status.F1 == 0)
        {
            button_status.F1 = 1;

            if (button_status.F0 == 0)
            {
                button_status.F0 = 1;
                Glcd_Write_Text("EIN ",26,4,0);
            }
            else
            {
                button_status.F0 = 0;
                Glcd_Write_Text("AUS",26,4,0);
            }
        }
    }
    else
    {
        button_status.F1 = 0;
    }

    // Schritt 3: Taste 2
    if ((x >= KONSTTASTE2X1) && (x <= KONSTTASTE2X2) && (y >= KONSTTASTE2Y1) && (y <= KONSTTASTE2Y2))
    {
        if (button_status.F2 == 0)
        {
            button_status.F2 = 1;

            if (button_status.F3 == 0)
            {
                button_status.F3 = 1;
                Glcd_Write_Text("EIN ",86,4,0);
            }
            else
            {
                button_status.F3 = 0;
                Glcd_Write_Text("AUS",86,4,0);
            }
        }
    }
    else
    {
        button_status.F2 = 0;
    }
}

/****************** Hauptprogramm ******************************************************/

/***************************************************************************************/
/* Aufgaben des Hauptprogramms:                                                        */
/*   + Controller initialisieren (Unterprogramm INIT)                                  */
/*   + Grafik-Display initialisieren und loeschen                                      */
/*   + 2 Tasten am Display darstellen                                                  */
/*   + Taetigkeiten/Unterprogramme, die alle zyklisch durchgefuehrt werden muessen:    */
/*      + alle 100 ms: Touchpanel abfragen, ob eine "virtuelle" Taste gedrueckt wurde  */
/*        (Unterprogramm TouchPanelTastenRoutine)                                      */
/***************************************************************************************/
void main(void)
{
    // Controller initialisieren
    Init();

    // Grafik-Display initialisieren
    Glcd_Init(&PORTB, 5, 4, 3, 2, 0, 1, &PORTD);
    Glcd_Fill(0);                      // Display loeschen
    Glcd_Set_Font(font5x7,5,7,32);
    Glcd_Fill(0);

    // Button 1
    Glcd_Rectangle(8,16,60,48,1);
    Glcd_Box(10,18,58,46,1);
    Glcd_Write_Text("TASTE 1",14,3,0);
    Glcd_Write_Text("AUS",26,4,0);
    
    // Button 2
    Glcd_Rectangle(68,16,120,48,1);
    Glcd_Box(70,18,118,46,1);
    Glcd_Write_Text("TASTE 2",74,3,0);
    Glcd_Write_Text("AUS",86,4,0);

    // Endlosschleife
    while(1)
    {
        if (FlagZeitbasis100ms)
        {
            // Touchpanel abfragen, ob eine "virtuelle" Taste gedrueckt wurde
            TouchPanelTastenRoutine();
            
            // Botschaftsflag (fuer 100ms-Zeitbasis) zuruecksetzen
            FlagZeitbasis100ms = 0;
        }
    }
}