/***************************************************************************************/
/* Unterprogramme zum Lesen des PIC-internen ADC-Moduls                                */
/*                                                                                     */
/* Compiler: mikroC                                                                    */
/*                                                                                     */
/* Entwickler: Stefan Buchgeher                                                        */
/* Entwicklungsbeginn der Software: 24. Juli 2009                                      */
/* Funktionsfaehig seit: 24. Juli 2009                                                 */
/* Letzte Bearbeitung: 24. Juli 2009                                                   */
/***************************************************************************************/

/****************** Include-Dateien ****************************************************/
#include "myPIC_ADC.H"

/****************** Unterprogramm zur Analog-Digital-Wandlung **************************/

/***************************************************************************************/
/* ADC8:                                                                               */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   Analog-Digital-Wandlung mit dem PIC-interne ADC fuer den ausgewaehlten ADC-Kanal  */
/*   starten, auf das Wandlungsergebnis warten und dieses dem aufrufenden Programm     */
/*   zurueckgeben.                                                                     */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   kanal:                                                                            */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   8 hoeherwertigen Bits der AD-Wandlung (PIC-Register: ADRESH)                      */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*   + ADC-Kanal auswaehlen (Bits <3:5> im Register ADCON0). Dazu muss die Kanal-Nummer*/
/*     an diese Position geschoben werden und mit dem Register ADCON0 verknuepft werden*/
/*   + ca. 50 Mikrosekunden (us) warten. Diese Zeit benoetigt der PIC um den internen  */
/*     Kondensator mit der zu messenden Analogspannung zu laden.                       */
/*   + Analog-Digital-Wandlung starten. Dazu das Flag GO (im Register ADCON0) setzen.  */
/*   + Warten, bis der PIC mit der Wandlung fertig ist. Der PIC setzt das Flag GO auto-*/
/*     matisch zurueck, wenn er mit der Analog-Digital-Wandlung fertig ist.            */
/*   + Aus den Registern ADRESL und ADRESH das Ergebnis zusammensetzen und an das auf- */
/*     rufende Unterprogramm (oder Hauptprogramm) zurueckgeben                         */
/*                                                                                     */
/* Anmerkung:                                                                          */
/*   Das Ausgabeformat der ADC-Wandlung, also wie die 10 Ergebnisbits in den Registern */
/*   ADRESL und ADRESH abgelegt werden wird an anderer Stelle (z.B. im Unterprogramm   */
/*   INIT) konfiguriert.                                                               */
/***************************************************************************************/
char ADC8(char kanal)
{
    ADCON0 = ADCON0 & 0b11000111;
    kanal = kanal << 3;
    ADCON0 = ADCON0 | kanal;

    Delay_us(50);

    ADCON0.GO = 1;
    while(ADCON0.GO);

    return (ADRESH);
}